<?php
/*-----------------------------------------------------------------------------
 * WWDAS Web API Class
 * 
 * An interface class to the WWDAS liveaboard trip scheduling system.  Allows 
 * a programatic method of making queries to get:
 * 
 * - A list of operating vessels. 
 * - A list of trips scheduled above one or all vessels in a given period.
 * - Trip and availability details for a specific trip.
 * 
 */
	
class WebAPIClass {
	
	private static $api_key				= "";									//  API key
	private static $api_server			= "webapi.masterliveaboards.com";		//  Default FQDN of API server
	private static $api_port 			= 80;									//  API server port (default = 80)
	private static $status 				= TRUE;
	private static $error_msg			= "";		
	private static $api_init_complete 	= FALSE;								// Flag when init is complete.

	
	/*-------------------------------------------------------------------------
	 * Function: __construct()
	 * 
	 * Initialize the class
	 * 
	*/
	public function __construct( $api_key="" ) {
		
		//  Ensure that Curl is installed
		//
		if ( !function_exists('curl_version')) {
			$this->error_msg = "Curl not installed.";
			$this->status = FALSE;
		}
		else {
			$this->init( $api_key );
			$this->status = TRUE;
			$this->api_server = "webapi.wwdas.com";
			$this->api_version = "1.0";
		}
		
		return( $this->status );
	}
	
	
	
	
	/*-------------------------------------------------------------------------
	 * Function: getVesselList()
	 * 
	 * Retrieve a list of vessels operating from today forward.  Vessels which 
	 * have ceased operation are not listed.
	 * 
	*/
	public function getVesselList() {
		
		//  Arguments to be passed to the server.
		//
		$post_request = array(
			'request'=>'vessel_list'
		);
		
		//  Make the call to retrieve the list of servers.
		//
		$vessel_list = $this->postRequest( $post_request );
		
		return( $vessel_list );
	}
	
	
	
	/*-------------------------------------------------------------------------
	 * Function: getTripAvailability()
	 * 
	 * Returns availability information for a given trip. 
	 * 
	 * Parameters: 
	 * 
	 * $trip_code : The WWDAS trip code for the requested trip.  This can be 
	 * 				retrieved from data returned by getVesselTrips().
	 * 
	 * Returned information: 
	 * 
	 * 		- trip code
	 * 		- vessel name
	 * 		- itinerary name
	 * 		- start date of trip
	 * 		- end date of trip
	 * 		- number of operating nights
	 * 		- trip price per person in Euros
	 * 		- number of spaces currently available. 
	 * 
	*/
	public function getTripAvailability( $trip_code ) {
		
		$post_request = array( 
				'request'=>'trip_availability',
				'trip_code'=>$trip_code
		);
		
		$trip_availability = $this->postRequest( $post_request );
		
		return( $trip_availability );
	}
	
	
	/*-------------------------------------------------------------------------
	 * Function: getVesselTrips()
	 * 
	 * Returns a list of trips scheduled on $vessel and departing between 
	 * $start_date and $end_date.
	 * 
	 * Parameters :
	 * 
	 * $vessel		: The name of the required vessel.  If not provided then  
	 * 				  trips on all vessels within the specified period will be 
	 * 				  returned.
	 * 
	 * $start_date	: Earliest start date to be considered.
	 * 
	 * $end_date	: Latest start date to be considered.
	 * 
	 * 
	 * Returns :
	 * 
	 * 		A list of trips with the following information for each trip:
	 * 
	 * 		- Vessel name
	 * 		- Trip code
	 *  	- Start date
	 *  	- End date
	 *  	- Nights on board
	 *  	- Itinerary name
	 *  	- Price in Euros
	 *  	- Number of spaces currently available.
	 *
	 */
	public function getVesselTrips( $vessel="", $start_date="", $end_date="" ) {
		
		$post_request = array( 
			'request'=>'vessel_trips',
			'vessel_name'=>$vessel, 
			'start_date'=>$start_date, 
			'end_date'=>$end_date
		);
		
		$trip_list = $this->postRequest( $post_request ); 
		
		return( $trip_list );
	}
	
	
	
	/*-------------------------------------------------------------------------
	 * Function: postRequest()
	 * 
	 * Formats a request and send to the API server.  This is an internal function
	 * used by meta-functions within the API client.
	 * 
	 */ 
	protected function postRequest( $a_post_vars="" ) {
		
		//  If $a_post_vars is empty, turn it into an empty array.
		//
		$a_post_vars = ( $a_post_vars == "" )
				? array()
				: (array)$a_post_vars;
		
		//  Correctly format the URL of the API server.
		//
		$url = "http://{$this->api_server}";
		
		//  Add some required parameters to the list of POST vars.
		//
		$a_post_vars[ 'api_key'] = $this->api_key;
		$a_post_vars[ 'api_version'] = $this->api_version;
		
		//  Build a proper POST query string out of the list of variables
		//  provided.
		//   
		$post_query = http_build_query( $a_post_vars );
		
		//  Create and initialize the Curl instance
		//
		$ch = curl_init( );
		curl_setopt( $ch, CURLOPT_URL, $url );
		curl_setopt( $ch, CURLOPT_POST, 1 );
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post_query );
		
		//  Make the query to the API server
		//
		$query_result = curl_exec ($ch);
		
		//  Close the Curl instance.
		//
		curl_close ($ch);

		$xml = simplexml_load_string( $query_result );
		
		return( $xml );
	}

	
	
	
	/*-------------------------------------------------------------------------
	 * Function: init()
	 * 
	 * Initialize the API
	 * 
	*/
	protected function init( $api_key ) {
		
		$this->api_key = $api_key;
		
		//  Flag the initialization as complete.
		//
		$this->api_init_complete = TRUE;
		
	}
	
	
	/*-------------------------------------------------------------------------
	 * Function: checkInit()
	 * 
	 * Check if the API has been initialized.   
	 * 
	 */
	protected function initCheck() {
		
		//  If the API is not initialized set up an error message 
		//
		if ( ! $this->api_init_complete ) {
			$this->setError( "API not initialized.");
		}
		
		return( $this->status);
	}
	
	
	/*-------------------------------------------------------------------------
	 * Function: setError()
	 * 
	 * Set the error message text and set the status flag to FALSE to indicate 
	 * that an error has occurred.
	 * 
	 */
	protected function setError( $error_message ) {
		$this->error_msg = $error_message;
		$this->status = FALSE;
	}
	
	
	/*-------------------------------------------------------------------------
	 * Function: setServer()
	 * 
	 * Manually set the API server.
	 * 
	 */
	public function setServer( $server_fqdn ) {
		$this->api_server = $server_fqdn;
		return( $this->status );
	}
	
	
	
	/*-------------------------------------------------------------------------
	 * Function: setPort()
	 * 
	 * Manually set the API server port.
	 * 
	 */
	protected function setPort( $server_port ) {
		$this->api_port = $server_port;
		return( $this->status );
	}
	
		
}
	?>